@@ -1,8 +1,12 @@
 use crate::bounding_volume::BoundingSphere;
 use crate::math::{Isometry, Real};
-use crate::shape::HeightField;
+use crate::shape::{GenericHeightField, HeightFieldCellStatus, HeightFieldStorage};
 
-impl HeightField {
+impl<Heights, Status> GenericHeightField<Heights, Status>
+where
+    Heights: HeightFieldStorage<Item = Real>,
+    Status: HeightFieldStorage<Item = HeightFieldCellStatus>,
+{
     /// Computes the world-space bounding sphere of this height-field, transformed by `pos`.
     #[inline]
     pub fn bounding_sphere(&self, pos: &Isometry<Real>) -> BoundingSphere {
