@@ -1,7 +1,7 @@
 //! Definition of the triangle shape.
 
 use crate::math::{Isometry, Point, Real, Vector};
-use crate::shape::SupportMap;
+use crate::shape::{FeatureId, SupportMap};
 use crate::shape::{PolygonalFeature, Segment};
 use crate::utils;
 
@@ -13,8 +13,9 @@ use std::mem;
 
 /// A triangle shape.
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "cuda", derive(cust_core::DeviceCopy))]
+#[derive(PartialEq, Debug, Copy, Clone, Default)]
 #[repr(C)]
-#[derive(PartialEq, Debug, Copy, Clone)]
 pub struct Triangle {
     /// The triangle first point.
     pub a: Point<Real>,
@@ -90,6 +91,17 @@ impl TrianglePointLocation {
     }
 }
 
+/// Orientation of a triangle.
+#[derive(Copy, Clone, Debug, PartialEq, Eq)]
+pub enum TriangleOrientation {
+    /// Orientation with a clockwise orientaiton, i.e., with a positive signed area.
+    Clockwise,
+    /// Orientation with a clockwise orientaiton, i.e., with a negative signed area.
+    CounterClockwise,
+    /// Degenerate triangle.
+    Degenerate,
+}
+
 impl From<[Point<Real>; 3]> for Triangle {
     fn from(arr: [Point<Real>; 3]) -> Self {
         *Self::from_array(&arr)
@@ -133,11 +145,21 @@ impl Triangle {
         ]
     }
 
+    /// Computes a scaled version of this triangle.
+    pub fn scaled(self, scale: &Vector<Real>) -> Self {
+        Self::new(
+            na::Scale::from(*scale) * self.a,
+            na::Scale::from(*scale) * self.b,
+            na::Scale::from(*scale) * self.c,
+        )
+    }
+
     /// Returns a new triangle with vertices transformed by `m`.
     #[inline]
     pub fn transformed(&self, m: &Isometry<Real>) -> Self {
         Triangle::new(m * self.a, m * self.b, m * self.c)
     }
+
     /// The three edges scaled directions of this triangle: [B - A, C - B, A - C].
     #[inline]
     pub fn edges_scaled_directions(&self) -> [Vector<Real>; 3] {
@@ -404,6 +426,24 @@ impl Triangle {
         // )
     }
 
+    /// Is this triangle degenerate or almost degenerate?
+    #[cfg(feature = "dim3")]
+    pub fn is_affinely_dependent_eps(&self, eps: Real) -> bool {
+        let p1p2 = self.b - self.a;
+        let p1p3 = self.c - self.a;
+        relative_eq!(
+            p1p2.cross(&p1p3).norm(),
+            0.0,
+            epsilon = eps * p1p2.norm().max(p1p3.norm())
+        )
+
+        // relative_eq!(
+        //     self.area(),
+        //     0.0,
+        //     epsilon = EPS * self.perimeter()
+        // )
+    }
+
     /// Tests if a point is inside of this triangle.
     pub fn contains_point(&self, p: &Point<Real>) -> bool {
         let p1p2 = self.b - self.a;
@@ -425,6 +465,49 @@ impl Triangle {
             && d13 >= 0.0
             && d13 <= p3p1.norm_squared()
     }
+
+    /// The normal of the given feature of this shape.
+    pub fn feature_normal(&self, _: FeatureId) -> Option<Unit<Vector<Real>>> {
+        self.normal()
+    }
+
+    /// The orientation of the triangle, based on its signed area.
+    ///
+    /// Returns `TriangleOrientation::Degenerate` if the triangle’s area is
+    /// smaller than `epsilon`.
+    #[cfg(feature = "dim2")]
+    pub fn orientation(&self, epsilon: Real) -> TriangleOrientation {
+        let area2 = (self.b - self.a).perp(&(self.c - self.a));
+        // println!("area2: {}", area2);
+        if area2 > epsilon {
+            TriangleOrientation::CounterClockwise
+        } else if area2 < -epsilon {
+            TriangleOrientation::Clockwise
+        } else {
+            TriangleOrientation::Degenerate
+        }
+    }
+
+    /// The orientation of the 2D triangle, based on its signed area.
+    ///
+    /// Returns `TriangleOrientation::Degenerate` if the triangle’s area is
+    /// smaller than `epsilon`.
+    pub fn orientation2d(
+        a: &na::Point2<Real>,
+        b: &na::Point2<Real>,
+        c: &na::Point2<Real>,
+        epsilon: Real,
+    ) -> TriangleOrientation {
+        let area2 = (b - a).perp(&(c - a));
+        // println!("area2: {}", area2);
+        if area2 > epsilon {
+            TriangleOrientation::CounterClockwise
+        } else if area2 < -epsilon {
+            TriangleOrientation::Clockwise
+        } else {
+            TriangleOrientation::Degenerate
+        }
+    }
 }
 
 impl SupportMap for Triangle {
@@ -505,15 +588,6 @@ impl ConvexPolyhedron for Triangle {
         }
     }
 
-    fn feature_normal(&self, _: FeatureId) -> Unit<Vector<Real>> {
-        if let Some(normal) = self.normal() {
-            // FIXME: We should be able to do much better here.
-            normal
-        } else {
-            Vector::y_axis()
-        }
-    }
-
     fn support_face_toward(
         &self,
         m: &Isometry<Real>,
